from dataclasses import dataclass
from enum import Enum
from typing import TypeAlias

from starlite import (AbstractAuthenticationMiddleware, ASGIConnection,
                      AuthenticationResult, DefineMiddleware,
                      NotAuthorizedException, Request)


class Group(Enum):
    """
    The possible groups a user could have
    """

    Participant = "participant"
    Sponsor = "sponsor"
    MLH = "mlh"
    Organizer = "organizer"
    Director = "director"


@dataclass(frozen=True)
class CurrentUser:
    """
    The user information for the request
    """

    id: int
    first_name: str
    last_name: str
    email: str
    group: Group
    context: str

    @property
    def is_admin(self):
        return self.context == "admin"

    @property
    def is_registering(self):
        return self.context == "manage"


class AuthenticationMiddleware(AbstractAuthenticationMiddleware):
    async def authenticate_request(
        self, connection: ASGIConnection
    ) -> AuthenticationResult:
        def value_or_fail(header: str) -> str:
            value = connection.headers.get(header)
            if value is None:
                raise NotAuthorizedException()

            return value

        try:
            user = CurrentUser(
                id=int(value_or_fail("X-Identity-ID")),
                first_name=value_or_fail("X-Identity-First-Name"),
                last_name=value_or_fail("X-Identity-Last-Name"),
                email=value_or_fail("X-Identity-Email"),
                group=Group(value_or_fail("X-Identity-Group")),
                context=value_or_fail("X-Identity-Context"),
            )
        except ValueError:
            raise NotAuthorizedException()

        return AuthenticationResult(user=user, auth=None)


AuthenticatedRequest: TypeAlias = Request[CurrentUser, None]

middleware = DefineMiddleware(AuthenticationMiddleware)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['identity_sdk']

package_data = \
{'': ['*']}

install_requires = \
['starlite>=1.45.1,<2.0.0']

setup_kwargs = {
    'name': 'identity-sdk',
    'version': '0.1.0',
    'description': "The Python SDK for The Hacker App's identity platform",
    'long_description': "# Identity SDK\n\nThe Python SDK for The Hacker App's identity platform.\n",
    'author': 'Alex Krantz',
    'author_email': 'alex@krantz.dev',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
